@@ -41,6 +41,7 @@
 #include "nestHome.h"
 #include "printtext.h"
 #include "readline.h"
+#include "spell.h"
 #include "strHand.h"
 #include "theme.h"
 
@@ -93,6 +94,15 @@ static struct tagConfDefValues {
 	{ "sasl_password",             TYPE_STRING,  1, "" },
 	{ "sasl_x509",                 TYPE_STRING,  1, "client.pem" },
 
+	{ "socks",                     TYPE_BOOLEAN, 2, "no" },
+	{ "socks_atyp",                TYPE_INTEGER, 1, "0" },
+	{ "socks_host",                TYPE_STRING,  1, "localhost" },
+	{ "socks_port",                TYPE_STRING,  1, "9050" },
+
+	{ "spell",                     TYPE_BOOLEAN, 2, "yes" },
+	{ "spell_lang",                TYPE_STRING,  1, "en_US" },
+	{ "spell_syswide",             TYPE_BOOLEAN, 1, "yes" },
+
 	{ "chanserv_host",             TYPE_STRING,  1, "services." },
 	{ "nickserv_host",             TYPE_STRING,  1, "services." },
 	{ "qbot_host",                 TYPE_STRING,  1, "CServe.quakenet.org" },
@@ -185,18 +195,6 @@ get_string(const char *name, const char *type)
 	return str;
 }
 
-static bool
-got_hits(const char *search_var)
-{
-	FOREACH_CDV() {
-		if (!strncmp(search_var, cdv_p->setting_name,
-		    strlen(search_var)))
-			return true;
-	}
-
-	return false;
-}
-
 static void
 hInstall(const char *name, const char *value)
 {
@@ -292,6 +290,15 @@ set_value_for_setting_ok_hook(const char *setting, const char *value)
 	if (strings_match(setting, "mouse") ||
 	    strings_match(setting, "mouse_events"))
 		readline_mouse_init();
+	else if (strings_match(setting, "spell_lang") ||
+	    strings_match(setting, "spell_syswide")) {
+#ifdef HAVE_HUNSPELL
+		spell_deinit();
+		spell_init(true);
+#else
+		/* null */;
+#endif
+	}
 	UNUSED_PARAM(value);
 }
 
@@ -365,34 +372,33 @@ write_config_header(FILE *fp)
 
 /* -------------------------------------------------- */
 
+static void
+add_name(PTEXTBUF matches, const char *name)
+{
+	if (textBuf_size(matches) == 0) {
+		if ((errno = textBuf_ins_next(matches, NULL, name, -1)) != 0)
+			err_sys("%s: textBuf_ins_next", __func__);
+	} else {
+		if ((errno = textBuf_ins_next(matches, textBuf_tail(matches),
+		    name, -1)) != 0)
+			err_sys("%s: textBuf_ins_next", __func__);
+	}
+}
+
 PTEXTBUF
 get_list_of_matching_settings(const char *search_var)
 {
-	PTEXTBUF matches;
-
-	if (!got_hits(search_var))
-		return NULL;
-
-	matches = textBuf_new();
+	PTEXTBUF matches = textBuf_new();
 
 	FOREACH_CDV() {
 		if (!strncmp(search_var, cdv_p->setting_name,
-		    strlen(search_var))) {
-			if (textBuf_size(matches) == 0) {
-				if ((errno = textBuf_ins_next(matches, NULL,
-				    cdv_p->setting_name, -1)) != 0) {
-					err_sys("%s: textBuf_ins_next",
-					    __func__);
-				}
-			} else {
-				if ((errno = textBuf_ins_next(matches,
-				    textBuf_tail(matches), cdv_p->setting_name,
-				    -1)) != 0) {
-					err_sys("%s: textBuf_ins_next",
-					    __func__);
-				}
-			}
-		}
+		    strlen(search_var)))
+			add_name(matches, cdv_p->setting_name);
+	}
+
+	if (textBuf_size(matches) == 0) {
+		textBuf_destroy(matches);
+		return NULL;
 	}
 
 	return matches;
