@@ -45,6 +45,7 @@
 #endif
 
 #if WIN32
+#include <io.h>
 #include <process.h> /* _getpid() */
 #endif
 
@@ -83,16 +84,21 @@
 /* Things with external linkage
    ============================ */
 
-chararray_t	g_swircVersion	= "v3.3.9";
+chararray_t	g_swircVersion	= "v3.4.1";
 chararray_t	g_swircYear	= "2012-2023";
 chararray_t	g_swircAuthor	= "Markus Uhlin";
 chararray_t	g_swircWebAddr	= "https://www.nifty-networks.net/swirc/";
 
-char *g_progname = const_cast<char *>("");
-long int g_pid = -1;
+FILE		*g_dev_null = NULL;
+char		*g_progname = const_cast<char *>("");
+char		*g_progpath = NULL;
+long int	 g_pid = -1;
 
-SETLOCALE_FN xsetlocale = NULL;
-char g_locale[200] = { '\0' };
+int	 g_stderr_fd = -1;
+int	 g_stdout_fd = -1;
+
+SETLOCALE_FN	 xsetlocale = NULL;
+char		 g_locale[200] = { '\0' };
 
 std::vector<std::string> g_join_list;
 
@@ -573,6 +579,7 @@ main(int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 #elif defined(WIN32)
+#define PEM_FILE "trusted_roots.pem"
 	char	*pgm = NULL,
 		*str = NULL;
 
@@ -584,6 +591,7 @@ main(int argc, char *argv[])
 			return EXIT_FAILURE;
 		}
 		*cp = '\0';
+		g_progpath = sw_strdup(str);
 		while ((cp = strchr(str, SLASH_CHAR)) != NULL)
 			*cp = '/';
 	}
@@ -591,6 +599,10 @@ main(int argc, char *argv[])
 		err_ret("bindtextdomain");
 		return EXIT_FAILURE;
 	}
+	if (str)
+		g_ca_file = strdup_printf("%s/%s", str, PEM_FILE);
+	else
+		g_ca_file = sw_strdup(PEM_FILE);
 	free(str);
 #endif
 	(void) bind_textdomain_codeset("swirc", "UTF-8");
@@ -637,6 +649,8 @@ main(int argc, char *argv[])
 	term_init();
 	nestHome_init();
 
+	if ((g_dev_null = xfopen(DEV_NULL, "w")) == NULL)
+		debug("dev null error");
 #if WIN32
 	if (pgm)
 		debug("_get_pgmptr: \"%s\"", pgm);
@@ -709,6 +723,7 @@ main(int argc, char *argv[])
 #endif
 
 	enter_io_loop();
+	free_and_null(&g_progpath);
 
 	/*
 	 * Reverse order...
@@ -776,3 +791,16 @@ cmdline_options_destroy(void)
 
 	delete g_cmdline_opts;
 }
+
+void
+redir_stderr(void)
+{
+	g_stderr_fd = dup(fileno(stderr));
+	(void) dup2(fileno(g_dev_null), fileno(stderr));
+}
+
+void
+restore_stderr(void)
+{
+	(void) dup2(g_stderr_fd, fileno(stderr));
+}
