@@ -69,7 +69,7 @@
 #define WATTR_ON(win, attrs)  ((void) wattr_on(win, attrs, NULL))
 #define WCOLOR_SET(win, cpn)  ((void) wcolor_set(win, cpn, NULL))
 
-#define STRLEN_CAST(string) strlen(reinterpret_cast<const char *>(string))
+#define STRLEN_CAST(string) strlen(reinterpret_cast<CSTRING>(string))
 
 /****************************************************************
 *                                                               *
@@ -255,7 +255,7 @@ addmbs(WINDOW *win, const unsigned char *mbs)
 	while ((c = *p++) != '\0')
 		WADDCH(win, c);
 #elif defined(WIN32)
-	(void) waddnstr(win, reinterpret_cast<const char *>(mbs), -1);
+	(void) waddnstr(win, reinterpret_cast<CSTRING>(mbs), -1);
 #endif
 }
 
@@ -320,30 +320,39 @@ case_bold(WINDOW *win, bool *is_bold)
 static unsigned char *
 convert_wc(wchar_t wc)
 {
-	const size_t	 size = (MB_LEN_MAX + 1);
+	const size_t	 size = MB_LEN_MAX;
 	mbstate_t	 ps;
-	size_t		 bytes_written;
-	unsigned char	*mbs = static_cast<unsigned char *>(xcalloc(size, 1));
+	size_t		 bytes_written = 0;
+	unsigned char	*mbs = static_cast<unsigned char *>(xmalloc(size + 1));
 
 	BZERO(&ps, sizeof(mbstate_t));
+	mbs[size] = '\0';
 
 #ifdef HAVE_BCI
-	if ((errno = wcrtomb_s(&bytes_written, reinterpret_cast<char *>(mbs),
+	if ((errno = wcrtomb_s(&bytes_written, reinterpret_cast<STRING>(mbs),
 	    size, wc, &ps)) != 0) {
 		err_log(errno, "printtext: %s: wcrtomb_s", __func__);
-		BZERO(mbs, size);
+		*mbs = '\0';
+		return mbs;
 	}
 #else
-	if ((bytes_written = wcrtomb(reinterpret_cast<char *>(mbs), wc, &ps)) ==
+	if ((bytes_written = wcrtomb(reinterpret_cast<STRING>(mbs), wc, &ps)) ==
 	    g_conversion_failed) {
 		err_log(EILSEQ, "printtext: %s: wcrtomb", __func__);
-		BZERO(mbs, size);
+		*mbs = '\0';
+		return mbs;
 	}
 #endif
 
-	UNUSED_VAR(bytes_written);
-	return static_cast<unsigned char *>(xrealloc(mbs, STRLEN_CAST(mbs) +
-	    1));
+	mbs[bytes_written] = '\0';
+	return static_cast<unsigned char *>(xrealloc(mbs, bytes_written + 1));
+}
+
+static inline void
+putbyte(char *cp, const unsigned char b)
+{
+	cp[0] = static_cast<char>(b);
+	cp[1] = '\0';
 }
 
 /***************************************************
@@ -362,12 +371,13 @@ check_for_part1(wchar_t **bufp, char *fg)
 	mbs = convert_wc(**bufp);
 
 	if (STRLEN_CAST(mbs) != 1 || !sw_isdigit(*mbs)) {
-		(*bufp)--;
+		--(*bufp);
 		free(mbs);
 		return STOP_INTERPRETING;
 	}
 
-	sw_snprintf(fg, 2, "%c", *mbs);
+	fg[0] = static_cast<char>(*mbs);
+	fg[1] = '\0';
 	free(mbs);
 	return GO_ON;
 }
@@ -388,13 +398,13 @@ check_for_part2(wchar_t **bufp, char *fg, bool *has_comma)
 	mbs = convert_wc(**bufp);
 
 	if (STRLEN_CAST(mbs) != 1 || (!sw_isdigit(*mbs) && *mbs != ',')) {
-		(*bufp)--;
+		--(*bufp);
 		free(mbs);
 		return STOP_INTERPRETING;
 	}
 
 	if (sw_isdigit(*mbs))
-		sw_snprintf(fg, 2, "%c", *mbs);
+		putbyte(fg, *mbs);
 	else if (*mbs == ',')
 		*has_comma = true;
 	else
@@ -420,15 +430,15 @@ check_for_part3(wchar_t **bufp, bool *has_comma, bool fg_complete, char *bg)
 	mbs = convert_wc(**bufp);
 
 	if (STRLEN_CAST(mbs) != 1 || (*mbs != ',' && !sw_isdigit(*mbs))) {
-		(*bufp)--;
+		--(*bufp);
 		free(mbs);
 		return STOP_INTERPRETING;
 	} else if (*mbs == ',' && *has_comma) {
-		(*bufp)--;
+		--(*bufp);
 		free(mbs);
 		return STOP_INTERPRETING;
 	} else if (*mbs != ',' && fg_complete) {
-		(*bufp)--;
+		--(*bufp);
 		free(mbs);
 		return STOP_INTERPRETING;
 	}
@@ -436,7 +446,7 @@ check_for_part3(wchar_t **bufp, bool *has_comma, bool fg_complete, char *bg)
 	if (*mbs == ',')
 		*has_comma = true;
 	else if (sw_isdigit(*mbs))
-		sw_snprintf(bg, 2, "%c", *mbs);
+		putbyte(bg, *mbs);
 	else
 		sw_assert_not_reached();
 
@@ -460,16 +470,16 @@ check_for_part4(wchar_t **bufp, bool got_digit_bg, char *bg)
 	mbs = convert_wc(**bufp);
 
 	if (STRLEN_CAST(mbs) != 1 || !sw_isdigit(*mbs)) {
-		(*bufp)--;
+		--(*bufp);
 		free(mbs);
 		return STOP_INTERPRETING;
 	} else if (got_digit_bg) {
-		sw_snprintf(++bg, 2, "%c", *mbs);
+		putbyte(++bg, *mbs);
 		free(mbs);
 		return STOP_INTERPRETING;
 	}
 
-	sw_snprintf(bg, 2, "%c", *mbs);
+	putbyte(bg, *mbs);
 	free(mbs);
 	return GO_ON;
 }
@@ -490,19 +500,19 @@ check_for_part5(wchar_t **bufp, char *bg)
 	mbs = convert_wc(**bufp);
 
 	if (STRLEN_CAST(mbs) != 1 || !sw_isdigit(*mbs)) {
-		(*bufp)--;
+		--(*bufp);
 		free(mbs);
 		return STOP_INTERPRETING;
 	}
 
-	sw_snprintf(bg, 2, "%c", *mbs);
+	putbyte(bg, *mbs);
 	free(mbs);
 	return GO_ON;
 }
 
 static void
 map_color(short int *inout, const short int i, const short int colorMap_size,
-	  attr_t *attr_out)
+    attr_t *attr_out)
 {
 	switch (*inout) {
 	case COLOR_BLACK:
@@ -567,7 +577,7 @@ printtext_set_color(WINDOW *win, bool *is_color, short int num1, short int num2)
 }
 
 static void
-init_numbers(const char *fg, const char *bg, short int &num1, short int &num2)
+init_numbers(CSTRING fg, CSTRING bg, short int &num1, short int &num2)
 {
 	struct integer_context intctx("term_background", 0, 15, 1);
 
@@ -627,7 +637,7 @@ case_color(WINDOW *win, bool *is_color, wchar_t **bufp)
 	printtext_set_color(win, is_color, num1, num2);
 
 	if (has_comma && !(bg[0]))
-		(*bufp)--;
+		--(*bufp);
 }
 
 /**
@@ -674,13 +684,13 @@ start_on_a_new_row(const ptrdiff_t sum, const WINDOW *win)
  *
  * @param[in]     ctx          Context structure
  * @param[in,out] rep_count    "Represent" count
- * @param[out]    line_count   Line count
+ * @param[out]    lines_count  Lines count
  * @param[out]    insert_count Insert count
  * @return Void
  */
 static void
 case_default(const struct case_default_context *ctx, int *rep_count,
-	     int *line_count, int *insert_count)
+    int *lines_count, int *insert_count)
 {
 	unsigned char *mbs;
 
@@ -704,7 +714,7 @@ case_default(const struct case_default_context *ctx, int *rep_count,
 		new_row(ctx->win, insert_count, rep_count);
 
 		if (care_about_max_lines &&
-		    !(++ (*line_count) < ctx->max_lines)) {
+		    !(++ (*lines_count) < ctx->max_lines)) {
 			free(mbs);
 			return;
 		}
@@ -723,7 +733,7 @@ case_default(const struct case_default_context *ctx, int *rep_count,
 		new_row(ctx->win, insert_count, rep_count);
 
 		if (care_about_max_lines &&
-		    !(++ (*line_count) < ctx->max_lines)) {
+		    !(++ (*lines_count) < ctx->max_lines)) {
 			free(mbs);
 			return;
 		}
@@ -782,7 +792,7 @@ case_underline(WINDOW *win, bool *is_underline)
  * Get multibyte string length
  */
 static size_t
-get_mb_strlen(const char *s)
+get_mb_strlen(CSTRING s)
 {
 	const size_t ERR_CASE1 = static_cast<size_t>(-1);
 	const size_t ERR_CASE2 = static_cast<size_t>(-2);
@@ -809,10 +819,10 @@ get_mb_strlen(const char *s)
 }
 
 static void
-set_indent(int *indent, const char *fmt, ...)
+set_indent(int *indent, CSTRING fmt, ...)
 {
-	va_list ap;
-	char *str = NULL;
+	va_list		 ap;
+	char		*str = NULL;
 
 	va_start(ap, fmt);
 	str = strdup_vprintf(fmt, ap);
@@ -832,11 +842,8 @@ set_indent(int *indent, const char *fmt, ...)
  * @return Message components
  */
 static struct message_components *
-get_processed_out_message(
-	const char *unproc_msg,
-	enum message_specifier_type spec_type,
-	bool include_ts,
-	const char *srv_time)
+get_processed_out_message(CSTRING unproc_msg, enum message_specifier_type
+    spec_type, bool include_ts, CSTRING srv_time)
 {
 	struct message_components *pout =
 	    static_cast<struct message_components *>(xcalloc(sizeof *pout, 1));
@@ -845,7 +852,7 @@ get_processed_out_message(
 	pout->indent = 0;
 
 	if (include_ts) {
-		char *ts = NULL;
+		STRING ts = NULL;
 
 		if (srv_time)
 			ts = sw_strdup(srv_time);
@@ -972,10 +979,11 @@ get_processed_out_message(
  */
 #if WIN32
 static wchar_t *
-windows_convert_to_utf8(const char *buf)
+windows_convert_to_utf8(CSTRING buf)
 {
-	const int size = static_cast<int>(strlen(buf) + 1);
-	wchar_t *out = static_cast<wchar_t *>(xcalloc(size, sizeof *out));
+	const int	 size = static_cast<int>(strlen(buf) + 1);
+	wchar_t		*out = static_cast<wchar_t *>(xcalloc(size,
+			     sizeof *out));
 
 	if (MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, buf, -1, out,
 	    size) > 0)
@@ -994,7 +1002,7 @@ windows_convert_to_utf8(const char *buf)
  * @return A wide-character string, or NULL on error.
  */
 static wchar_t *
-try_convert_buf_with_cs(const char *buf, const char *codeset)
+try_convert_buf_with_cs(CSTRING buf, CSTRING codeset)
 {
 	char			*original_locale = NULL;
 	char			*tmp_locale = NULL;
@@ -1071,7 +1079,7 @@ static wchar_t *
 perform_convert_buffer(const char **in_buf)
 {
 	bool		 chars_lost = false;
-	const char	*ar[] = {
+	stringarray_t	 ar = {
 #if defined(UNIX)
 		"UTF-8", "utf8",
 		"ISO-8859-1", "ISO8859-1", "iso88591",
@@ -1181,7 +1189,7 @@ vprinttext_mutex_init(void)
 
 PPRINTTEXT_CONTEXT
 printtext_context_new(PIRC_WINDOW window, enum message_specifier_type spec_type,
-		      bool include_ts)
+    bool include_ts)
 {
 	PPRINTTEXT_CONTEXT ctx =
 	    static_cast<PPRINTTEXT_CONTEXT>(xcalloc(sizeof *ctx, 1));
@@ -1203,11 +1211,8 @@ printtext_context_destroy(PPRINTTEXT_CONTEXT ctx)
 }
 
 void
-printtext_context_init(
-	PPRINTTEXT_CONTEXT ctx,
-	PIRC_WINDOW window,
-	enum message_specifier_type spec_type,
-	bool include_ts)
+printtext_context_init(PPRINTTEXT_CONTEXT ctx, PIRC_WINDOW window,
+    enum message_specifier_type spec_type, bool include_ts)
 {
 	if (ctx == NULL) {
 		err_log(EINVAL, "%s", __func__);
@@ -1250,7 +1255,7 @@ color_pair_find(short int fg, short int bg)
  * space pointed to by @cp
  */
 void
-print_and_free(const char *msg, char *cp)
+print_and_free(CSTRING msg, char *cp)
 {
 #ifndef UNIT_TESTING
 	PRINTTEXT_CONTEXT ctx;
@@ -1271,7 +1276,7 @@ print_and_free(const char *msg, char *cp)
 }
 
 void
-printf_and_free(char *cp, const char *fmt, ...)
+printf_and_free(char *cp, CSTRING fmt, ...)
 {
 	PPRINTTEXT_CONTEXT ctx;
 	va_list ap;
@@ -1294,7 +1299,7 @@ printf_and_free(char *cp, const char *fmt, ...)
  * @return Void
  */
 void
-printtext(PPRINTTEXT_CONTEXT ctx, const char *fmt, ...)
+printtext(PPRINTTEXT_CONTEXT ctx, CSTRING fmt, ...)
 {
 	va_list ap;
 
@@ -1304,7 +1309,7 @@ printtext(PPRINTTEXT_CONTEXT ctx, const char *fmt, ...)
 }
 
 void
-printtext_print(const char *what, const char *fmt, ...)
+printtext_print(CSTRING what, CSTRING fmt, ...)
 {
 	PPRINTTEXT_CONTEXT ctx;
 	va_list ap;
@@ -1318,6 +1323,12 @@ printtext_print(const char *what, const char *fmt, ...)
 
 	if (what == NULL)
 		/* null */;
+	else if (strings_match(what, "sp1"))
+		ctx->spec_type = TYPE_SPEC1;
+	else if (strings_match(what, "sp2"))
+		ctx->spec_type = TYPE_SPEC2;
+	else if (strings_match(what, "sp3"))
+		ctx->spec_type = TYPE_SPEC3;
 	else if (strings_match(what, "err"))
 		ctx->spec_type = TYPE_SPEC1_FAILURE;
 	else if (strings_match(what, "success"))
@@ -1332,10 +1343,10 @@ printtext_print(const char *what, const char *fmt, ...)
 	printtext_context_destroy(ctx);
 }
 
-static char *get_buffer(const char *) PTR_ARGS_NONNULL;
+static char *get_buffer(CSTRING) PTR_ARGS_NONNULL;
 
 static char *
-get_buffer(const char *orig)
+get_buffer(CSTRING orig)
 {
 #ifdef HAVE_LIBICONV
 #define UTF8_MAXBYTE 4
@@ -1344,9 +1355,19 @@ get_buffer(const char *orig)
 
 	std::list<std::string> fromcode;
 
+#if defined(__cplusplus) && __cplusplus >= 201103L
+	fromcode.emplace_back("UTF-8");
+	fromcode.emplace_back("ISO-8859-1");
+	fromcode.emplace_back("ISO-8859-15");
+	fromcode.emplace_back("WINDOWS-1251");
+	fromcode.emplace_back("WINDOWS-1252");
+#else
 	fromcode.push_back("UTF-8");
 	fromcode.push_back("ISO-8859-1");
 	fromcode.push_back("ISO-8859-15");
+	fromcode.push_back("WINDOWS-1251");
+	fromcode.push_back("WINDOWS-1252");
+#endif
 
 	for (const std::string &x : fromcode) {
 		char *in, *orig_copy, *out, *out_p;
@@ -1406,13 +1427,13 @@ get_buffer(const char *orig)
  * @return Void
  */
 void
-printtext_puts(WINDOW *pwin, const char *buf, int indent, int max_lines,
-	       int *rep_count)
+printtext_puts(WINDOW *pwin, CSTRING buf, int indent, int max_lines,
+    int *rep_count)
 {
 	char		*tmpbuf = NULL;
 	const char	*tmpbuf_p = NULL;
 	int		 insert_count = 0;
-	int		 line_count = 0;
+	int		 lines_count = 0;
 	struct text_decoration_bools
 			 booleans; // calls constructor
 	wchar_t		*wc_buf = NULL;
@@ -1479,7 +1500,7 @@ printtext_puts(WINDOW *pwin, const char *buf, int indent, int max_lines,
 				diff = (wcp - wc_bufp);
 			struct case_default_context def_ctx(pwin, wc,
 			    !wcscmp(wc_bufp + 1, L""), indent, max_lines, diff);
-			case_default(&def_ctx, rep_count, &line_count,
+			case_default(&def_ctx, rep_count, &lines_count,
 			    &insert_count);
 			break;
 		} /* case default */
@@ -1488,7 +1509,7 @@ printtext_puts(WINDOW *pwin, const char *buf, int indent, int max_lines,
 		++ wc_bufp;
 
 		if (is_scrollok(pwin) && max_lines > 0)
-			max_lines_flagged = line_count >= max_lines;
+			max_lines_flagged = lines_count >= max_lines;
 	}
 
 	free(wc_buf);
@@ -1528,12 +1549,12 @@ set_timestamp(char *dest, size_t destsize,
  * @return Void
  */
 void
-vprinttext(PPRINTTEXT_CONTEXT ctx, const char *fmt, va_list ap)
+vprinttext(PPRINTTEXT_CONTEXT ctx, CSTRING fmt, va_list ap)
 {
-	char *fmt_copy = NULL;
-	const int tbszp1 = textBuf_size(ctx->window->buf) + 1;
-	struct integer_context intctx("textbuffer_size_absolute", 350, 4700,
-	    1000);
+	STRING			 fmt_copy = NULL;
+	const int		 tbszp1 = textBuf_size(ctx->window->buf) + 1;
+	struct integer_context	 intctx("textbuffer_size_absolute", 350, 4700,
+				     1000);
 	struct message_components *pout = NULL;
 
 #if defined(UNIX)
@@ -1587,7 +1608,7 @@ vprinttext(PPRINTTEXT_CONTEXT ctx, const char *fmt, va_list ap)
 	}
 
 	if (ctx->window->logging) {
-		char *logpath;
+		STRING logpath;
 
 		if ((logpath = log_get_path(g_server_hostname,
 		    ctx->window->label)) != NULL) {
