@@ -1,5 +1,5 @@
 /* Window functions
-   Copyright (C) 2012-2022 Markus Uhlin. All rights reserved.
+   Copyright (C) 2012-2023 Markus Uhlin. All rights reserved.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
@@ -101,6 +101,19 @@ static PIRC_WINDOW hash_table[200] = { NULL };
 
 /* -------------------------------------------------- */
 
+static void
+add_match(PTEXTBUF matches, const char *what)
+{
+	if (textBuf_size(matches) != 0) {
+		if ((errno = textBuf_ins_next(matches, textBuf_tail(matches),
+		    what, -1)) != 0)
+			err_sys("%s: textBuf_ins_next", __func__);
+	} else {
+		if ((errno = textBuf_ins_next(matches, NULL, what, -1)) != 0)
+			err_sys("%s: textBuf_ins_next", __func__);
+	}
+}
+
 /**
  * Apply window options
  */
@@ -165,7 +178,7 @@ first_page_up(PIRC_WINDOW window)
 		printtext_puts(tmp, element->text, element->indent, -1,
 		    &rep_count);
 		element = element->prev;
-		elt_count ++;
+		elt_count++;
 		i += rep_count;
 	} /* while */
 	(void) curs_set(1);
@@ -209,7 +222,7 @@ get_dynamic_scroll_amount(PIRC_WINDOW window, plus_minus_t pm)
 			element = element->next;
 		else
 			sw_assert_not_reached();
-		amount ++;
+		amount++;
 		i += rep_count;
 	} /* while */
 	(void) curs_set(1);
@@ -376,7 +389,7 @@ window_redraw(PIRC_WINDOW window, const int rows, const int pos,
 			printtext_puts(pwin, element->text, element->indent, -1,
 			    NULL);
 			element = element->next;
-			++ i;
+			++i;
 		}
 		(void) curs_set(1);
 		(void) atomic_swap_bool(&g_redrawing_window, false);
@@ -452,7 +465,7 @@ window_recreate(PIRC_WINDOW window, int rows, int cols)
 	 * draw main window
 	 */
 	if (window->scroll_mode) {
-		if (! (window->scroll_count > HEIGHT)) {
+		if (!(window->scroll_count > HEIGHT)) {
 			window->saved_size = 0;
 			window->scroll_count = 0;
 			window->scroll_mode = false;
@@ -549,6 +562,47 @@ window_by_refnum(int refnum)
 	return NULL;
 }
 
+PTEXTBUF
+get_list_of_matching_channels(const char *search_var)
+{
+	PTEXTBUF	matches = textBuf_new();
+
+	FOREACH_HASH_TABLE_ENTRY() {
+		FOREACH_WINDOW_IN_ENTRY() {
+			if (is_irc_channel(window->label) &&
+			    !strncasecmp(search_var, window->label,
+			    strlen(search_var)))
+				add_match(matches, window->label);
+		}
+	}
+	if (textBuf_size(matches) == 0) {
+		textBuf_destroy(matches);
+		return NULL;
+	}
+	return matches;
+}
+
+PTEXTBUF
+get_list_of_matching_queries(const char *search_var)
+{
+	PTEXTBUF	matches = textBuf_new();
+
+	FOREACH_HASH_TABLE_ENTRY() {
+		FOREACH_WINDOW_IN_ENTRY() {
+			if (!is_irc_channel(window->label) &&
+			    window != g_status_window &&
+			    !strncasecmp(search_var, window->label,
+			    strlen(search_var)))
+				add_match(matches, window->label);
+		}
+	}
+	if (textBuf_size(matches) == 0) {
+		textBuf_destroy(matches);
+		return NULL;
+	}
+	return matches;
+}
+
 /**
  * Change window to the one specified by @label
  */
@@ -752,7 +806,7 @@ window_scroll_down(PIRC_WINDOW window, const int amount)
 		window->scroll_count -= amount;
 	}
 
-	if (! (window->scroll_count > HEIGHT)) {
+	if (!(window->scroll_count > HEIGHT)) {
 		window->saved_size = 0;
 		window->scroll_count = 0;
 		window->scroll_mode = false;
